//+------------------------------------------------------------------+
//| Basket Profit 2 Pairs sq.mq4
//+------------------------------------------------------------------+

#property copyright "Squalou"
#property link      ""


/* 2-Pairs Basket Profit Indicator, by squalou:
 * draws a line representing the P/L of a 2-Pairs Basket, in Account Currency amount.
 * The light-blue vertical line is the Reference Line showing pointing the "Basket BE point",
 * typically put on the bar where you would "buy" this basket.
 * It can be moved manually to change the "Basket BE point", the indicator will automatically adjust the BE horizontal line (0.00 value)
 * accordingly at the next incoming tick;
 *
 * This was developped to help research on this Dreamliner's thread:
 * http://www.forexfactory.com/showthread.php?t=160912
 * and its EA dev thread from SteveHopwood:
 * http://www.forexfactory.com/showthread.php?t=278476
 * 
 */
 
/* - the Pair2 input will be updated automatically if the "Pair2" Label object on the chart is changed;
 * - the "Pair2" Label object is also used by the following Indicators/EAs to update their default "Pair2" input:
 *   - "Correlation Lines sq" indicator, 
 *   - "OverLayChart sq" indicator,
 *   - "Stochastic different pairs sq" indicator
 *   - "Stoch Diff 2 Pair EAsq" EA,
 * Changing the "Pair2" Label object text will automatically change the settings in those Indicators/EA if they are loaded on the chart.
 * Changing the "Pair2" input of any of these Indicators/EAs will have the same effect, and the "Pair2" Label object will be updated on the chart;
 *
 * - it is HIGHLY RECOMMENDED to load the indicators on the pair that has the highest tick refresh rate 
 *   to get the most reactive Indicators/EAs update -- especially when using an EA;
 * 
 * Note that the update of the Indicators/EAs will take effect only at the next incoming tick;
 * When markets are closed, it is therefore necessary to manually refresh the chart (right-click+Refresh),
 * or better use the "MT4_ticker.exe" program to generate artificial ticks;
 *
 * - You can load the "correlation stoch diff.tpl" template file on a chart to load all of the indicators above, but it will NOT load the EA;
 */


/* Mods:
  - 2010.02.15:
    - added "SMAPeriods" (100) and "StDevPeriods" (500) inputs:
      this will plot the SMA of the Basket Profit line, and the +1,+2,-1,-2 StdDev lines around the SMA.
      If the pairs are highly correlated, then,
      when the basket Profit line goes outside the -2/+2*SD zone, (pairs are getting out of sync),
      it is "expected" to "come back home" = return to mean (=SMA line).
*/

#property indicator_separate_window
//#property indicator_minimum -100
//#property indicator_maximum 100
#property indicator_levelstyle STYLE_SOLID
#property indicator_level1 0
#property indicator_level2 100
#property indicator_level3 -100
#property indicator_level4 200
#property indicator_level5 -200
#property indicator_buffers 6
#property indicator_color1 Gold       // Basket Profit Line
#property indicator_color2 Aqua       // SMA
#property indicator_color3 Green      // +1SD
#property indicator_color4 LimeGreen  // +2SD
#property indicator_color5 Orange     // -1SD
#property indicator_color6 Red        // -2SD

#property indicator_width2 2          // SMA


extern string Pair1             = "";
extern double Lot1              = 0.1; // <0 means SELL, >0 means BUY
extern double Spread1           = 0;
extern string _Pair2_help       = "Pair2 blank will be taken from \"Pair2\" chart object";
extern string Pair2             = "";
extern double Lot2              = -0.1; // <0 means SELL, >0 means BUY
extern double Spread2           = 0;
extern int DaysBackReference    = 0; // where to put the "Reference Point 0"
extern string _Lot_help         = "Lot<0 means SELL, Lot>0 means BUY";
extern bool   ShowEachPair      = true;
extern int    FontSize          = 17;
extern int    HLevelMax         = 200;
extern int    HLevelStep        = 10;
extern color  HLevelColor       = Maroon;
extern string  sd="----StDev Inputs----";
extern int     SMAPeriods       = 100;
extern int     StDevPeriods     = 500;

double buffer1[],buffer2[],buffer3[],_1_stddev_high[],_1_stddev_low[],_2_stddev_high[],_2_stddev_low[],ma[],dist_from_ma[];
string m = "";
bool stopped;

string ref_line = "Basket BE Reference";
datetime ref_time;
double ref1,ref2;
//double Point1,Point2;
double Value1,Value2;
//int pipMultTab[]={1,10,1,10,1,10,100}; // multiplier to convert pips to Points;

string s2Pairs,sPair1,sPair1withLot,sPair2,sPair2withLot,shortname;
int win;

//+------------------------------------------------------------------+
int init()
//+------------------------------------------------------------------+
{
  if (Digits==3 || Digits==5)
   {
      Spread1=Spread1*10;
      Spread2=Spread2*10;
   }
  string m = StringSubstr(Symbol(), 6, StringLen(Symbol()) - 6);
  if (Pair1 == "") Pair1 = Symbol();
  else Pair1 = StringSubstr(Pair1,0,6) + m;

  set_Pair2(Pair2);

  sPair1=""; sPair1withLot="";
  if (Lot1!=0) { sPair1withLot=DoubleToStr(Lot1,2)+"*"+Pair1; sPair1=Pair1; }
  sPair2=""; sPair2withLot="";
  if (Lot2!=0) {
    if (Lot2>0) string signPair2="+"; else signPair2="-";
    sPair2=signPair2+Pair2;
    sPair2withLot=signPair2+DoubleToStr(MathAbs(Lot2),2)+"*"+Pair2;
  }
  s2Pairs = sPair1withLot+sPair2withLot;

  shortname="Basket "+AccountCurrency()+" ("+s2Pairs+")";
  IndicatorShortName(shortname);
  win = WindowFind(shortname);

  // plot BE Reference Line on DaysBackReference at 00:00
  ref_time = iTime(NULL,PERIOD_D1,DaysBackReference);
  drawVLine(ref_line,ref_time,DodgerBlue,STYLE_SOLID);
  ref1 = iClose(Pair1,0,iBarShift(Pair1,0,ref_time));
  ref2 = iClose(Pair2,0,iBarShift(Pair2,0,ref_time));

  stopped=false;
  if (ref1==0) {
    Alert("!!! "+WindowExpertName()+" : no history data for "+Pair1+" !!!");
    stopped=true;
  }
  if (ref2==0) {
    Alert("!!! "+WindowExpertName()+" : no history data for "+Pair2+" !!!");
    stopped=true;
  }

  //---- indicators
  IndicatorBuffers(8);
  int b=0;
  SetIndexBuffer(b, buffer1); SetIndexLabel(b,"Basket "+AccountCurrency()+" ("+s2Pairs+")");
  //b++; SetIndexBuffer(b, buffer2); SetIndexLabel(b,"Basket "+AccountCurrency()+" ("+Pair1+")");
  //b++; SetIndexBuffer(b, buffer3); SetIndexLabel(b,"Basket "+AccountCurrency()+" ("+Pair2+")");
  b++; SetIndexBuffer(b, ma);             SetIndexLabel(b,"SMA("+SMAPeriods+")");
  b++; SetIndexBuffer(b, _1_stddev_high); SetIndexLabel(b,"+1*stddev");
  b++; SetIndexBuffer(b, _2_stddev_high); SetIndexLabel(b,"+2*stddev");
  b++; SetIndexBuffer(b, _1_stddev_low);  SetIndexLabel(b,"-1*stddev");
  b++; SetIndexBuffer(b, _2_stddev_low);  SetIndexLabel(b,"-2*stddev");
  b++; SetIndexBuffer(b, dist_from_ma);


  Value1 = PointValueFor1Lot(Pair1)/MarketInfo(Pair1,MODE_POINT);
  Value2 = PointValueFor1Lot(Pair2)/MarketInfo(Pair2,MODE_POINT);

  drawHLine(shortname+0, 0, HLevelColor^0x00FFFFFF, STYLE_SOLID, 0, win);
  for (int kk=HLevelStep;kk<=HLevelMax;kk+=HLevelStep) {
    drawHLine(shortname+kk, kk, HLevelColor, STYLE_DOT, 0, win);
    drawHLine(shortname+DoubleToStr(-kk,0), -kk, HLevelColor, STYLE_DOT, 0, win);
  }


  return (0);
}

//+------------------------------------------------------------------+
int deinit()
//+------------------------------------------------------------------+
{
  RemoveObjects(shortname);
  return (0);
}

//--------------------------------------------------------------------------------------
void RemoveObjects(string prefix)
//--------------------------------------------------------------------------------------
{   
   int i;
   string objname;

   for (i = ObjectsTotal(); i >= 0; i--) {
      objname = ObjectName(i);
      if (StringFind(objname, prefix, 0) > -1) ObjectDelete(objname);
   }
} /* RemoveObjects*/

//+------------------------------------------------------------------+
int start()
//+------------------------------------------------------------------+
{
  if (stopped) return;

  int BarsBack = Bars;// DaysBackReference*PERIOD_D1/Period();
  
  int i, counted_bars = IndicatorCounted();
  if (counted_bars > 0) counted_bars--;
  int limit = Bars - counted_bars;
  limit = MathMin(limit,BarsBack);

  if (get_Pair2_from_chart(Pair2)) { // will update itself if "Pair2" label object was changed (manually or from another indicator/EA)
    deinit();
    init();
    limit=BarsBack;
  }

  if (ObjectFind(ref_line)>=0) {  
    double new_ref = ObjectGet(ref_line,OBJPROP_TIME1);
    if (new_ref!=ref_time) {
      ref_time = new_ref;
      Print("BE Reference Line moved to "+TimeToStr(ref_time,TIME_DATE|TIME_MINUTES));
      drawVLine(ref_line,ref_time,DodgerBlue,STYLE_SOLID);
      ref1 = iClose(Pair1,0,iBarShift(Pair1,0,ref_time));
      ref2 = iClose(Pair2,0,iBarShift(Pair2,0,ref_time));
      limit=BarsBack;
    }
  }
  
  double price1,price2;
  for(i=limit; i>=0; i--)
  {
    price1 = (iClose(Pair1,0,iBarShift(Pair1,0,Time[i]))-ref1)*Value1*Lot1-(Spread1*MarketInfo(Pair1,MODE_TICKVALUE)*MathAbs(Lot1));
    price2 = (iClose(Pair2,0,iBarShift(Pair2,0,Time[i]))-ref2)*Value2*Lot2-(Spread2*MarketInfo(Pair2,MODE_TICKVALUE)*MathAbs(Lot2));
    buffer1[i] = price1+price2;

    double stddev;
    ma[i] = SMA(buffer1,SMAPeriods,i);
    dist_from_ma[i] = buffer1[i]-ma[i];
    stddev = StdDev(dist_from_ma,StDevPeriods, i);
    _1_stddev_high[i] = ma[i]+stddev;
    _1_stddev_low[i]  = ma[i]-stddev;
    _2_stddev_high[i] = ma[i]+2*stddev;
    _2_stddev_low[i]  = ma[i]-2*stddev;

  }

  drawFixedLbl(shortname+"_1",   sPair1,  0, 0,   15, FontSize, "Courrier", indicator_color2, false, win);
  drawFixedLbl(shortname+"_2",   sPair2,  0, 110, 15, FontSize, "Courrier", indicator_color3, false, win);
  drawFixedLbl(shortname+"_1+2", s2Pairs, 0, 0,   40, FontSize, "Courrier", indicator_color1, false, win);

  return (0);
}

/*+------------------------------------------------------------------+ */
double SMA(double t[], int n, int shift)
/*+------------------------------------------------------------------+ */
{ /* calc SMA of an array;*/
  double avg=0;
  int i;
  for (i=0;i<n;i++) avg += t[i+shift];
  avg/=n;
  return(avg);
}

/*+------------------------------------------------------------------+ */
double StdDev(double t[], int n, int shift)
/*+------------------------------------------------------------------+ */
{ /* calc StdDev of an array;*/
  /* - 68.2% of the data are within 1*StdDev of the avg value [avg-StdDev   .. avg+StdDev]*/
  /* - 95.4% of the data are within 2*StdDev of the avg value [avg-2*StdDev .. avg+2*StdDev]*/
  double avg=0, sum=0;
  int i;
  for (i=0;i<n;i++) avg += t[i+shift];
  avg/=n;
  for (i=0;i<n;i++) sum += (t[i+shift]-avg)*(t[i+shift]-avg);
  return(MathSqrt(sum/n));
}

//+------------------------------------------------------------------
void drawVLine(string objname, int time, color c, int style=STYLE_SOLID, int width=0, int win=0)
//+------------------------------------------------------------------
{
  ObjectCreate(objname, OBJ_VLINE, win, time, 0);
  ObjectSet(objname, OBJPROP_COLOR, c);
  ObjectSet(objname, OBJPROP_STYLE, style);
  ObjectSet(objname, OBJPROP_WIDTH, width);
}

//+------------------------------------------------------------------
void drawHLine(string name, double price, color c, int style=STYLE_SOLID, int width=0, int win=0)
//+------------------------------------------------------------------
{
  ObjectCreate(name, OBJ_HLINE, win, 0, price);
  ObjectSet(name, OBJPROP_COLOR, c);
  ObjectSet(name, OBJPROP_STYLE, style);
  ObjectSet(name, OBJPROP_WIDTH, width);
}

//+------------------------------------------------------------------+
string __m;
void set_Pair2(string &Pair2)
//+------------------------------------------------------------------+
{
  __m = StringSubstr(Symbol(), 6, StringLen(Symbol()) - 6);
  Pair2 = StringSubstr(Pair2,0,6) + __m;
  if (MarketInfo(Pair2,MODE_ASK)==0) {// Pair2 does not exist, get it from the chart "Pair2" object
    get_Pair2_from_chart(Pair2);
  } else {
    drawFixedLbl("Pair2", Pair2, 1, 8, 20, 14, "Arial", Magenta, false);
  }
}

//+------------------------------------------------------------------+
bool get_Pair2_from_chart(string &Pair2)
//+------------------------------------------------------------------+
{

  /* Import Pair2 input from the "Pair2" Label object if it exists on the chart
   * The "Pair2" Label is also used by "OverLayChart" indicator,
   * and the "Stochastic different pairs 1.6b.mq4" EA,
   * so that they can all be loaded on the same chart and run with the same second-pair
   */
  if (ObjectFind("Pair2")>=0) {
    string p2 = ObjectDescription("Pair2");
    if (p2!=Pair2) {
      Pair2 = StringSubstr(p2,0,6) + __m;
      if (MarketInfo(Pair2,MODE_ASK)==0) Pair2=Symbol();// Pair2 does not exist, default to chart's symbol
      drawFixedLbl("Pair2", Pair2, 1, 8, 20, 14, "Arial", Magenta, false);
      Print("Found \"Pair2\" object : "+Pair2);
      return(true);
    }
  } else { // "Pair2" object does not exist: create it
    Pair2 = StringSubstr(Pair2,0,6) + __m;
    if (MarketInfo(Pair2,MODE_ASK)==0) Pair2=Symbol();// Pair2 does not exist, default to chart's symbol
    drawFixedLbl("Pair2", Pair2, 1, 8, 20, 14, "Arial", Magenta, false);
  }
  return(false);
}

//--------------------------------------------------------------------------------------
void drawFixedLbl(string objname, string s, int Corner, int DX, int DY, int FSize, string Font, color c, bool bg, int win=0)
//--------------------------------------------------------------------------------------
{
   if (ObjectFind(objname) < 0) ObjectCreate(objname, OBJ_LABEL, win, 0, 0);
   
   ObjectSet(objname, OBJPROP_CORNER, Corner);
   ObjectSet(objname, OBJPROP_XDISTANCE, DX);
   ObjectSet(objname, OBJPROP_YDISTANCE, DY);
   ObjectSet(objname,OBJPROP_BACK, bg);      
   ObjectSetText(objname, s, FSize, Font, c);
} // drawFixedLbl

//+------------------------------------------------------------------+
double PointValueFor1Lot(string symbol)
//+------------------------------------------------------------------+
{
  // PtVal = TickValue * Point / TickSize;
  return ( MarketInfo(symbol,MODE_TICKVALUE)*MarketInfo(symbol,MODE_POINT)/MarketInfo(symbol,MODE_TICKSIZE) );
}
//end

